import _readOnlyError from "@babel/runtime/helpers/readOnlyError";
import _extends from "@babel/runtime/helpers/extends";
import vpatch from './vpatch.js';
import renderRect, { updateRect } from '../render/elements/renderRect';
import renderText, { updateText } from '../render/elements/renderText';
import renderImage, { updateImage } from '../render/elements/renderImage';
/**
 * 真实的 Dom 打补钉
 * @param {*} rootNode 实际的 DOM
 * @param {*} patches
 */

function patch(rootNode, patches, group) {
  var walker = {
    index: 0
  };
  dftWalk(rootNode, walker, patches, group);
}
/**
 * 深入优先遍历算法 (depth-first traversal，DFT)
 * @param {*} node
 * @param {*} walker
 * @param {*} patches
 */


function dftWalk(node, walker, patches, group) {
  var currentPatches = patches[walker.index] || []; // TEXT元素不往里遍历children，和diff算法的识别TEXT change逻辑保持一致

  node.type !== 'TEXT' && node.children && node.children.forEach(function (child) {
    walker.index++;
    dftWalk(child, walker, patches, group);
  });

  if (currentPatches && currentPatches.length > 0) {
    applyPatches(node, currentPatches, group);
  }
}

function applyPatches(node, currentPatches, group) {
  var targetElement = group.find(function (item) {
    return item.attr('name') === "g-react-item-" + node.name;
  });
  currentPatches.forEach(function (currentPatch) {
    if (targetElement || currentPatch.type === vpatch.INSERT) {
      var newNode = currentPatch.node;

      switch (currentPatch.type) {
        case vpatch.REMOVE:
          targetElement.remove();
          break;

        case vpatch.REPLACE:
          // node.parentNode.replaceChild(render(newNode), node);
          targetElement.remove();

          switch (newNode.type) {
            case 'TEXT':
              renderText(group)(newNode);
              break;

            case 'RECT':
              renderRect(group)(newNode);
              break;

            case 'IMAGE':
              renderImage(group)(newNode);
              break;

            default:
              break;
          }

          break;

        case vpatch.TEXT:
          if (node.parent && node.parent.type === 'RECT') {
            var parentElement = group.find(function (item) {
              return item.attr('name') === "g-react-item-" + node.parent.name;
            });
            var parentX = parentElement && parentElement.attrs.x || 0;
            var parentY = parentElement && parentElement.attrs.y || 0;
            setTimeout(function () {
              // 需要根据父框left和top来做一次相对定位
              var newParentElement = group.find(function (item) {
                return item.attr('name') === "g-react-item-" + node.parent.name;
              });
              var newParentX = newParentElement && newParentElement.attrs.x || 0;
              var newParentY = newParentElement && newParentElement.attrs.y || 0;
              newNode.box.left += newParentX - parentX;
              newNode.box.top += newParentY - parentY; // newNode.box.left = 0;
              // newNode.box.tpo = 0;

              updateText(targetElement)(newNode);
            }, 30);
          }

          break;

        case vpatch.PROPS:
          setProps(node, currentPatch.props, targetElement, group);
          break;

        case vpatch.INSERT:
          switch (newNode.type) {
            case 'TEXT':
              if (newNode.parent && newNode.parent.type === 'RECT') {
                setTimeout(function () {
                  // 需要根据父框left和top来做一次相对定位
                  var newParentElement = group.find(function (item) {
                    return item.attr('name') === "g-react-item-" + node.parent.name;
                  });

                  if (newParentElement) {
                    var newParentX = newParentElement.attrs.x;
                    var newParentY = newParentElement.attrs.y;
                    newNode.box.left += newParentX;
                    newNode.box.top += newParentY;
                  }

                  renderText(group)(newNode);
                }, 10);
              } else {
                renderText(group)(newNode);
              }

              break;

            case 'RECT':
              renderRect(group)(newNode);
              break;

            case 'IMAGE':
              renderImage(group)(newNode);
              break;

            default:
              break;
          }

          break;

        case vpatch.REORDER:
          // reorderChildren(node, currentPatch.moves);
          break;

        default:
          break;
      }
    }
  });
}
/**
 * 设置真实 Dom 属性值
 * @param {*} node
 * @param {*} props
 */


function setProps(node, props, targetElement, group) {
  var newProps = {};
  ['props', 'box'].forEach(function (item) {
    newProps = _extends({}, newProps, props[item] || {});
  });
  var newStyle = {};

  if (Object.keys(props.style || {}).length > 0) {
    // 有style变化
    newStyle = _extends({}, node.style, props.style);
  } // TODO 父元素有xy变化时 需要根据新的xy与老xy的差值来定位子元素。但如果父xy未变更，则直接以xy来定位


  if (node.type === 'RECT') {
    if ((newProps.left || newProps.top) && node.children && node.children.length > 0) {
      // 涉及RECT定位变更，则children为TEXT的元素跟随移动
      node.children.forEach(function (child) {
        if (child.type === 'TEXT') {
          var childTextTarget = group.find(function (item) {
            return item.attr('name') === "g-react-item-" + child.name;
          });

          if (childTextTarget) {
            var x = childTextTarget.attrs.x || 0,
                y = childTextTarget.attrs.y || 0;

            if (newProps.left) {
              x += newProps.left - (targetElement.attrs.x || 0);
            }

            if (newProps.top) {
              y += newProps.top - (targetElement.attrs.y || 0);
            }

            childTextTarget.attr({
              x: x,
              y: y
            });
          }
        }
      });
    }

    updateRect(targetElement)(newProps, newStyle);
  } else if (node.type === 'IMAGE') {
    updateImage(targetElement)(newProps, newStyle, node);
  }
}
/**
 * reorderChildren 处理 list diff render
 * @param {*} domNode
 * @param {*} moves
 */


function reorderChildren(domNode, moves) {
  for (var i = 0; i < moves.removes.length; +i, _readOnlyError("i")) {
    var index = moves.removes[i].index;
    var node = domNode.childNodes[index];
    domNode.removeChild(node);
  }

  for (var j = 0; j < moves.inserts.length; +j, _readOnlyError("j")) {
    var _moves$inserts$j = moves.inserts[j],
        _index = _moves$inserts$j.index,
        _node = _moves$inserts$j.node;
    domNode.insertBefore(_node, _index === domNode.childNodes.length ? null : childNodes[_index]);
  }
}

export default patch;