"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports["default"] = void 0;

var _lodash = require("lodash");

var _vpatch = _interopRequireDefault(require("./vpatch"));

/**
 * 二叉树 diff
 * @param lastVNode
 * @param newVNode
 */
function diff(lastVNode, newVNode) {
  var walker = {
    index: 0
  };
  var patches = {}; // patches.old = lastVNode

  dftWalk(lastVNode, newVNode, walker, patches);
  return patches;
}
/**
 * 深入优先遍历算法 (depth-first traversal，DFT)
 * @param {*} lastVNode
 * @param {*} newVNode
 * @param {*} index
 * @param {*} patches
 */


function dftWalk(lastVNode, newVNode, walker, patches) {
  // 为保持当前步骤index不被children的diff而改变记错步骤
  var index = walker.index;

  if (lastVNode === newVNode) {
    return;
  }

  var currentPatch = []; // Node is removed

  if (newVNode === null || newVNode === undefined) {
    currentPatch.push({
      type: _vpatch["default"].REMOVE,
      node: lastVNode
    }); // diff text
  } else if (lastVNode.type === 'TEXT' && newVNode.type === 'TEXT') {
    if ((0, _lodash.get)(newVNode, 'children[0].value') !== (0, _lodash.get)(lastVNode, 'children[0].value')) {
      currentPatch.push({
        type: _vpatch["default"].TEXT,
        node: newVNode
      });
    } // same node  此处会出行，parentNode 先 moves 处理，childnode 再做一次处理（替换或修改属性）

  } else if (newVNode.type === lastVNode.type // && newVNode.key === lastVNode.key
  ) {
    // newVNode.key === lastVNode.key 才会执行 diffChildren
    if (newVNode.name === lastVNode.name) {
      var propsPatches = diffProps(lastVNode, newVNode);

      if (Object.keys(propsPatches).length > 1) {
        currentPatch.push({
          type: _vpatch["default"].PROPS,
          props: propsPatches
        });
      }

      if (lastVNode.children && lastVNode.children.length > 0 || newVNode.children && newVNode.children.length > 0) {
        diffChildren(lastVNode.children, newVNode.children, currentPatch, walker, patches);
      }
    } else {
      currentPatch.push({
        type: _vpatch["default"].REPLACE,
        node: newVNode
      });
    } // Nodes are not the same

  } else {
    currentPatch.push({
      type: _vpatch["default"].REPLACE,
      node: newVNode
    });
  }

  if (currentPatch.length) {
    if (patches[index]) {
      patches[index].push(currentPatch);
    } else {
      patches[index] = currentPatch;
    }
  }
}

function diffChildren(lastChildren, newChildren, apply, walker, patches) {
  var orderedSet = reorder(lastChildren, newChildren);
  var len = lastChildren.length > newChildren.length ? lastChildren.length : newChildren.length;

  for (var i = 0; i < len; i++) {
    if (!lastChildren[i]) {
      // insert node
      if (newChildren[i] && !orderedSet.moves) {
        apply.push({
          type: _vpatch["default"].INSERT,
          node: newChildren[i]
        });
      }
    } else {
      walker.index++;
      dftWalk(lastChildren[i], newChildren[i], walker, patches);
    }
  } // console.error('orderedSet.moves', orderedSet.moves);


  if (orderedSet.moves) {
    apply.push(orderedSet);
  }
}
/**
 * diff vnode props
 * @param {*} lastProps
 * @param {*} newProps
 */


function diffProps(lastVNode, newVNode) {
  var propsPatches = {
    props: {},
    box: {},
    style: {},
    type: newVNode.type
  };
  ['props', 'box', 'style'].forEach(function (item) {
    // Find out diff props
    var last = lastVNode[item];
    var current = newVNode[item];

    for (var key in last) {
      if (current[key] !== last[key]) {
        propsPatches[item][key] = current[key];
      }
    } // Find out new props


    for (var _key in current) {
      if (!last.hasOwnProperty(_key)) {
        propsPatches[item][_key] = current[_key];
      }
    }
  });
  var availableProps = {};

  for (var key in propsPatches) {
    if (Object.keys(propsPatches[key]).length > 0) {
      availableProps[key] = propsPatches[key];
    }
  }

  return availableProps;
}
/**
 * List diff, naive left to right reordering
 * @param {*} lastChildren
 * @param {*} newChildren
 *
 * 原理利用中间数组 simulate， remove得到子集、insert 插入操作完成
 * 例 oldList [1,4,6,8,9] newList [0,1,3,5,6]
 * 转换拿到中间数组按老数组索引 [1, null, 6, null, null ]
 * remove null 得到子集 [1, 6]
 * insert 插入完成
 */


function reorder(lastChildren, newChildren) {
  var lastMap = keyIndex(lastChildren);
  var lastKeys = lastMap.keys;
  var lastFree = lastMap.free;

  if (lastFree.length === lastChildren.length) {
    return {
      moves: null
    };
  }

  var newMap = keyIndex(newChildren);
  var newKeys = newMap.keys;
  var newFree = newMap.free;

  if (newFree.length === newChildren.length) {
    return {
      moves: null
    };
  } // simulate list to manipulate


  var children = [];
  var freeIndex = 0;

  for (var i = 0; i < lastChildren.length; i++) {
    var item = lastChildren[i];

    if (item.key) {
      if (newKeys.hasOwnProperty('key')) {
        var itemIndex = newKeys[item.key];
        children.push(newChildren[itemIndex]);
      } else {
        children.push(null);
      }
    } else {
      var _itemIndex = newFree[freeIndex++];
      children.push(newChildren[_itemIndex] || null);
    }
  }

  var simulate = children.slice();
  var removes = [];
  var inserts = [];
  var j = 0; // remove  value is null and  no key property

  while (j < simulate.length) {
    if (simulate[j] === null || !simulate[j].hasOwnProperty('key')) {
      var _patch = remove(simulate, j);

      removes.push(_patch);
    } else {
      j++;
    }
  }

  console.error('simulate', simulate);

  for (var _i = 0; _i < newChildren.length; _i++) {
    var wantedItem = newChildren[_i];
    var simulateItem = simulate[_i];

    if (wantedItem.key) {
      if (simulateItem && wantedItem.key !== simulateItem.key) {
        // key property is not equal, insert, simulateItem add placeholder
        inserts.push({
          type: _vpatch["default"].INSERT,
          index: _i,
          node: wantedItem
        });
        simulateItem.splice(_i, 1);
      }
    } else {
      // no key property, insert, simulateItem add placeholder
      inserts.push({
        type: _vpatch["default"].INSERT,
        index: _i,
        node: wantedItem
      });
      simulateItem && simulateItem.splice(_i, 1);
    }
  }

  return {
    type: _vpatch["default"].REORDER,
    moves: {
      removes: removes,
      inserts: inserts
    }
  };
}

function remove(arr, index) {
  arr.splice(index, 1);
  return {
    type: _vpatch["default"].REMOVE,
    index: index
  };
}
/**
 * Convert list to key-item keyIndex object.
 * @param {*} children
 * convert [{id: "a", key: 'a'}, {id: "b", key: 'b'}, {id: "a"}]
 * result { keys: { a: 0, b: 1}, free: [ 2 ] }
 */


function keyIndex(children) {
  var keys = {};
  var free = [];
  var length = children.length;

  for (var i = 0; i < length; i++) {
    var child = children[i];

    if (child.key) {
      keys[child.key] = i;
    } else {
      free.push(i);
    }
  }

  return {
    keys: keys,
    // A hash of key name to index
    free: free // An array of unkeyed item indices

  };
}

var _default = diff;
exports["default"] = _default;